<?php
// $Id: customfilter.admin.inc,v 1.1.2.8.2.40 2009/08/21 14:01:07 kiam Exp $

/**
 * @file
 * Administration page callbacks for Custom filter.
 */

/**
 * @addtogroup customfilter
 * @{
 */

/*****************************************************************************
 * Menu callbacks / form builders, submit/validate functions.
 ****************************************************************************/

/**
 * Return the export form.
 */
function customfilter_export_form(&$form_state) {
  $form = array();
  $filters = _customfilter_get_filters();
  $step = isset($form_state['storage']['step']) ? $form_state['storage']['step'] : 1;

  switch ($step) {
    case 1:
      if ($filters) {
        foreach ($filters as $filter) {
          $opt[$filter['fid']] = check_plain($filter['name']);
        }

        $form['filters'] = array(
          '#type' => 'radios',
          '#title' => t('Filters'),
          '#description' => 'Choose the filter to export.',
          '#options' => $opt,
        );

        $form['submit'] = array(
          '#type' => 'submit',
          '#value' => 'Export',
        );
      }
      else {
        $form['filters'] = array(
          '#value' => '<p>' . t('There are no custom filters defined.') . '</p>',
        );
      }
      break;

    case 2:
      $export_data = array();
      $fid = $form_state['values']['filters'];

      if ($filter = $filters[$fid]) {
        $export_data[] = '$filter = array(';
        $export_data[] = "  'fid' => _customfilter_map_filter_id('". $filter['fid'] ."'),";
        $export_data[] = "  'name' => base64_decode('". base64_encode($filter['name']) ."'),";
        $export_data[] = "  'cache' => {$filter['cache']},";
        $export_data[] = "  'description' => base64_decode('". base64_encode($filter['description']) ."'),";
        $export_data[] = "  'shorttips' => base64_decode('". base64_encode($filter['shorttips']) ."'),";
        $export_data[] = "  'longtips' => base64_decode('". base64_encode($filter['longtips']) ."'),";
        $export_data[] = ");";
        $export_data[] = '';

        $rules = array();
        _customfilter_rules_tree($rules, $fid, 0);
        foreach ($rules as $rule) {
          $export_data[] = '$rules[] = array(';
          $export_data[] = "  'rid' => _customfilter_map_rule_id('". $rule['rid'] ."'),";
          $export_data[] = "  'fid' => _customfilter_map_filter_id('". $filter['fid'] ."'),";
          $export_data[] = "  'prid' => _customfilter_map_rule_id('". $rule['prid'] ."'),";
          $export_data[] = "  'name' => base64_decode('". base64_encode($rule['name']) ."'),";
          $export_data[] = "  'description' => base64_decode('". base64_encode($rule['description']) ."'),";
          $export_data[] = "  'enabled' => {$rule['enabled']},";
          $export_data[] = "  'matches' => base64_decode('". base64_encode($rule['matches']) ."'),";
          $export_data[] = "  'pattern' => base64_decode('". base64_encode($rule['pattern']) ."'),";
          $export_data[] = "  'replacement' => base64_decode('". base64_encode($rule['replacement']) ."'),";
          $export_data[] = "  'code' => {$rule['code']},";
          $export_data[] = "  'weight' => {$rule['weight']},";
          $export_data[] = ");";
        }
      }

      if ($rows = count($export_data)) {
        $export_data = implode("\n", $export_data);

        $form['export'] = array(
          '#type' => 'textarea',
          '#title' => t('Export data'),
          '#description' => t('Copy the export text and paste it into the import form.'),
          '#cols' => 60,
          '#default_value' => $export_data,
          '#rows' => max(40, $rows),
        );
      }
      else {
        $form['export'] = array(
          '#value' => '<p>' . t('The selected filter has not been found in the database table.') . '</p>',
        );
      }
      break;
  }

  return $form;
}

function customfilter_export_form_submit($form, &$form_state) {
  $step = isset($form_state['storage']['step']) ? $form_state['storage']['step'] : 1;
  if ($step < 2) {
    $form_state['storage']['step'] = $step + 1;
  }
}

/**
 * Return the filter deleting form.
 */
function customfilter_filter_delete($form_state, $fid) {
  _customfilter_breadcrumb();

  $filters = _customfilter_get_filters();

  $form['#fid'] = $fid;

  return confirm_form(
    $form, t('Are you sure you want to delete %name?', array('%name' => $filters[$fid]['name'])),
    'admin/settings/customfilter',
    NULL,
    t('Delete')
  );
}

/**
 * Submission callback for the filter deleting form.
 */
function customfilter_filter_delete_submit($form, &$form_state) {
  $fid = $form['#fid'];

  db_query('DELETE FROM {customfilter_rules} WHERE fid = %d', $fid);
  db_query('DELETE FROM {customfilter_filters} WHERE fid = %d', $fid);
  customfilter_cache_clear($form_state['values']['fid']);

  $form_state['redirect'] = 'admin/settings/customfilter';
}

/**
 * Return the filter editing form.
 */
function customfilter_filter_edit($form_state, $op, $fid = 0) {
  if ($op == 'edit') {
    $filters = _customfilter_get_filters();
    $item = $filters[$fid];

    _customfilter_breadcrumb();
  }
  elseif ($op == 'add') {
    $fid = (integer) db_result(db_query("SELECT MAX(fid) FROM {customfilter_filters}")) + 1;
    $item = array(
      'fid' => $fid,
      'name' => t('Filter #!fid', array('!fid' => $fid)),
      'cache' => 0,
      'description' => '',
      'shorttips' => '',
      'longtips' => '',
    );
  }

  $form['fid'] = array(
    '#type' => 'value',
    '#value' => $item['fid'],
  );

  $form['operation'] = array(
    '#type' => 'value',
    '#value' => $op,
  );

  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#description' => t('The name of the filter.'),
    '#default_value' => $item['name'],
    '#required' => TRUE,
  );

  $form['cache'] = array(
    '#type' => 'checkbox',
    '#title' => t('Cache'),
    '#description' => t('If checked, the content will be cached.'),
    '#default_value' => $item['cache'],
  );

  $form['description'] = array(
    '#type' => 'textarea',
    '#title' => t('Description'),
    '#description' => t('A description of the purpose of this filter.'),
    '#default_value' => $item['description'],
  );

  $form['shorttips'] = array(
    '#type' => 'textarea',
    '#title' => t('Tips (short)'),
    '#default_value' => $item['shorttips'],
    '#rows' => 5,
  );

  $form['longtips'] = array(
    '#type' => 'textarea',
    '#title' => t('Tips (full)'),
    '#default_value' => $item['longtips'],
    '#rows' => 20,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}

/**
 * Submission callback for the filter editing form.
 */
function customfilter_filter_edit_submit($form, &$form_state) {
  switch ($form_state['values']['operation']) {
    case 'edit':
      drupal_write_record('customfilter_filters', $form_state['values'], 'fid');
      break;

    case 'add':
      drupal_write_record('customfilter_filters', $form_state['values']);
      break;
  }

  customfilter_cache_clear($form_state['values']['fid']);

  $form_state['redirect'] = 'admin/settings/customfilter';
}

/**
 * Filters list page.
 */
function customfilter_filters_list() {
  $header = array(
    t('Name'),
    t('Description'),
    array('data' => t('Operations'), 'colspan' => '3'),
  );

  $rows = array();

  foreach (_customfilter_get_filters() as $id => $filter) {
    $rows[] = array(
      l($filter['name'], "admin/settings/customfilter/{$id}"),
      '<em>' . $filter['description'] . '</em>',
      l(t('delete'), "admin/settings/customfilter/{$id}/delete"),
      l(t('edit'), "admin/settings/customfilter/{$id}/edit"),
      l(t('test'), "admin/settings/customfilter/{$id}/test"),
    );
  }

  $output = theme(
    'table',
    $header,
    $rows ? $rows : array(array(array('data' => t('No filters have been defined.'), 'colspan' => 6)))
  );

  return $output;
}

/**
 * Return the import form.
 */
function customfilter_import_form() {
  $form = array();

  $form['import'] = array(
    '#type' => 'textarea',
    '#title' => t('Import data'),
    '#description' => t('Paste the text you obtained from the export form.'),
    '#cols' => 60,
    '#default_value' => '',
    '#rows' => 40,
    '#required' => TRUE,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Import'),
  );

  return $form;
}

function customfilter_import_form_submit($form, &$form_state) {
  @eval($form_state['values']['import']);

  if (!isset($filter) || !isset($rules) || !is_array($filter) || !is_array($rules)) {
    form_set_error('import', t('The import data is not valid import text.'));
    return;
  }

  drupal_write_record('customfilter_filters', $filter);

  foreach ($rules as $rule) {
    drupal_write_record('customfilter_rules', $rule);
  }

  customfilter_cache_clear();
}

/**
 * Autocomplete callback.
 */
function customfilter_node_autocomplete($string = '') {
  $matches = array();

  if ($string) {
    $result = db_query_range("SELECT title FROM {node} WHERE title LIKE '%s%%'", $string, 0, 10);

    while ($node = db_fetch_object($result)) {
      $matches[$node->title] = check_plain($node->title);
    }
  }

  drupal_json($matches);
}

/**
 * Return the list of replacement rules form.
 */
function customfilter_rules_list(&$form_state, $fid, $rid = 0) {
  $enabled = array(
    'default' => array(),
    'options' => array(),
  );
  $form = array('#tree' => TRUE);
  $rules = _customfilter_get_rules($fid, $rid);

  if (count($rules)) {
    $form['fid'] = array(
      '#type' => 'hidden',
      '#value' => $fid,
    );

    _customfilter_rules_tree_form($form, $enabled, $rules, 0);

    $form['enabled'] = array(
      '#type' => 'checkboxes',
      '#options' => $enabled['options'],
      '#default_value' => $enabled['default'],
      '#checkall' => 'customfilter-rules-list-enabled',
    );

    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save'),
    );
  }

  _customfilter_breadcrumb();

  return $form;
}

/**
 * Submission callback for the list of replacement rules form.
 */
function customfilter_rules_list_submit($form, &$form_state) {
  $rule = new stdClass();
  $rule->fid = $form_state['values']['fid'];

  foreach ($form_state['values']['rid'] as $rid) {
    $rule->rid = $rid;
    $rule->prid = $form_state['values']['prid'][$rid];
    $rule->enabled = $form_state['values']['enabled'][$rid];
    $rule->weight = $form_state['values']['weight'][$rid];

    drupal_write_record('customfilter_rules', $rule, 'rid');
    $done = TRUE;
  }

  if (isset($done)) {
    drupal_set_message(t('The configuration options have been saved.'));
    customfilter_cache_clear($form_state['values']['fid']);
  }

  $form_state['redirect'] = "admin/settings/customfilter/{$form_state['values']['fid']}";
}

/**
 * Return the replacement rule delete form.
 */
function customfilter_rule_delete($form_state, $rid) {
  $filters = _customfilter_get_filters();
  $rule = db_fetch_object(db_query("SELECT * FROM {customfilter_rules} WHERE rid = %d", $rid));
  $rules_count = db_result(db_query("SELECT COUNT(*) FROM {customfilter_rules} WHERE prid = %d", $rid));

  if (isset($filters[$fid])) {
    _customfilter_breadcrumb(
      array(
        l($filters[$rule->fid]['name'], "admin/settings/customfilter/{$rule->fid}"),
      )
    );
  }

  $form['#rid'] = $rid;
  $form['#fid'] = $rule->fid;

  if ($rules_count) {
    $msg_text .= " <p>" . t('This rule has subrules. If you delete this rule, they will be deleted too.') . "</p>";
  }

  $msg_text .= t('This action cannot be undone.');

  return confirm_form(
    $form, t('Are you sure you want to delete the replacement rule %name?', array('%name' => $rule->name)),
    "admin/settings/customfilter/{$rule->fid}",
    $msg_text,
    t('Delete')
  );
}

/**
 * Submission callback for the replacement rule delete form.
 */
function customfilter_rule_delete_submit($form, &$form_state) {
  _customfilter_delete_rule($form['#rid']);
  customfilter_cache_clear($form_state['values']['fid']);

  $form_state['redirect'] = "admin/settings/customfilter/{$form['#fid']}";
}

/**
 * Return the replacement rule edit form.
 */
function customfilter_rule_edit($form_state, $op, $fid, $rid = 0) {
  $filters = _customfilter_get_filters();

  if (isset($filters[$fid])) {
    _customfilter_breadcrumb(
      array(
        l($filters[$fid]['name'], "admin/settings/customfilter/{$fid}"),
      )
    );
  }

  if ($op == 'edit') {
    $item = db_fetch_array(db_query("SELECT * FROM {customfilter_rules} WHERE rid = %d", $rid));
  }
  elseif ($op == 'add') {
    $item = array(
      'rid' => (integer) db_result(db_query("SELECT MAX(rid) FROM {customfilter_rules}")) + 1,
      'prid' => $rid,
      'fid' => $fid,
      'name' => '$1',
      'description' => '',
      'enabled' => 1,
      'matches' => 1,
      'pattern' => '/regex/i',
      'replacement' => 'Regular Expressions',
      'code' => 0,
      'weight' => 0,
    );
  }

  $matchopt = drupal_map_assoc(range(0, 99));

  $form['rid'] = array(
    '#type' => 'value',
    '#value' => $item['rid'],
  );

  $form['fid'] = array(
    '#type' => 'value',
    '#value' => $fid,
  );

  $form['prid'] = array(
    '#type' => 'value',
    '#value' => $item['prid'],
  );

  $form['operation'] = array(
    '#type' => 'value',
    '#value' => $op,
  );

  if ($item['prid'] != 0) {
    $form['matches'] = array(
      '#type' => 'select',
      '#title' => t('# Match'),
      '#description' => t('n-th matched substring in parent rule. This replacement rule will replace only for that substring.'),
      '#options' => $matchopt,
      '#default_value' => $item['matches'],
    );
  }

  $form['name'] = array(
    '#type' => 'textfield',
    '#description' => t('The name of this replacement rule.'),
    '#title' => t('Name'),
    '#default_value' => $item['name'],
    '#required' => TRUE,
  );

  $form['weight'] = array(
    '#title' => t('Weight'),
    '#description' => t('The rules with lower weight will be executed first.'),
    '#default_value' => $item['weight'],
  );

  $form['enabled'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enabled'),
    '#default_value' => $item['enabled'],
  );

  $form['description'] = array(
    '#type' => 'textarea',
    '#title' => t('Description'),
    '#description' => t('The description of this rule.'),
    '#default_value' => $item['description'],
  );

  $form['pattern'] = array(
    '#type' => 'textarea',
    '#title' => t('Pattern'),
    '#description' => t('The regular expression to use. Look at <a href="http://www.php.net/manual/en/regexp.reference.php">Regular Expression Details</a> for more help.'),
    '#default_value' => $item['pattern'],
    '#rows' => 3,
  );

  $form['code'] = array(
    '#type' => 'checkbox',
    '#title' => t('PHP Code'),
    '#description' => t('If selected, the replacement text is PHP code.'),
    '#default_value' => $item['code'],
  );

  $form['replacement']['replacement'] = array(
    '#type' => 'textarea',
    '#title' => t('Replacement text'),
    '#description' => t('The replacement text that will replace the matched string. Use $n (i.e. $1, $25) or ${n} (i.e. ${1}, ${25}), with n range from 0 to 99, to get the n-th original strings matched ($0 represents the entire matched string). If you select <strong>PHP Code</strong>, you can enter PHP code that will be executed during the elaboration of the rules. n-th matched string is provided in <code>$matches[n]</code>, and there is a global variable <code>$customfilter</code> you can use to store values that will be kept during the execution of different rules of the same filter. PHP code must set a value for <code>$result</code>, and must not be entered between <code><</code><code>?php ?></code>. Note that executing incorrect PHP-code can break your Drupal site.'),
    '#default_value' => $item['replacement'],
    '#rows' => 16,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}

/**
 * Submission callback for the replacement rule edit form.
 */
function customfilter_rule_edit_submit($form, &$form_state) {
  switch ($form_state['values']['operation']) {
    case 'edit':
      drupal_write_record('customfilter_rules', $form_state['values'], 'rid');
      break;

    case 'add':
      drupal_write_record('customfilter_rules', $form_state['values']);
      break;
  }

  customfilter_cache_clear($form_state['values']['fid']);

  $form_state['redirect'] = "admin/settings/customfilter/{$form_state['values']['fid']}";
}

function customfilter_test_form($form_state, $fid) {
  $filters = _customfilter_get_filters();
  $form = array();
  $options = array();

  if (isset($filters[$fid])) {
    _customfilter_breadcrumb(
      array(
        l($filters[$fid]['name'], "admin/settings/customfilter/{$fid}"),
      )
    );

    if (isset($form_state['storage']['output'])) {
      $output = $form_state['storage']['output'];

      $form['result'] = array(
        '#type' => 'fieldset',
        '#title' => t('Result'),
        '#collapsible' => TRUE,
      );

      $form['result']['result-output'] = array(
        '#value' => check_plain($output),
        '#prefix' => '<div>',
        '#suffix' => '</div>'
      );

      if (function_exists('_filter_htmlcorrector')) {
        $form['result']['preview'] = array(
          '#type' => 'fieldset',
          '#title' => t('Preview'),
          '#collapsible' => TRUE,
          '#collapsed' => TRUE,
        );

        if (($preview = _filter_htmlcorrector($output)) != $output) {
          $form['result']['preview']['message'] = array(
            '#value' => t('The result returned from the filter contains HTML tags left open.'),
            '#prefix' => '<div class="messages warning">',
            '#suffix' => '</div>'
          );
        }

        $form['result']['preview']['preview-output'] = array(
          '#value' => $preview,
          '#prefix' => '<div>',
          '#suffix' => '</div>'
        );
      }
    }

    foreach ($filters as $id => $filter) {
      $options[$id] = $filter['name'];
    }

    if (!empty($options)) {
      $form['filter'] = array(
        '#type' => 'select',
        '#title' => t('Filter to test'),
        '#options' => $options,
        '#default_value' => $fid,
      );

      $form['trial'] = array(
        '#type' => 'fieldset',
        '#title' => t('Trial text'),
      );

      if (db_result(db_query("SELECT COUNT(*) FROM {node}"))) {
        $form['trial']['#description'] = t('Enter the title of the node, or the text to use to test the filter.');

        $form['trial']['node'] = array(
          '#type' => 'textfield',
          '#title' => t('Node title'),
          '#default_value' => '',
          '#autocomplete_path' => 'customfilter/node/autocomplete',
        );
      }
      else {
        $form['trial']['#description'] = t('Enter the text to use to test the filter.');
      }

      $text = isset($form_state['storage']['input']) ? $form_state['storage']['input'] : '';
      $rows = count(preg_split("/(\r\n?|\n)/", $text));

      $form['trial']['text'] = array(
        '#type' => 'textarea',
        '#default_value' => $text,
        "#rows" => max(15, $rows),
      );

      $form['op'] = array(
        '#type' => 'submit',
        '#value' => t('Test')
      );
    }
    else {
      drupal_set_message(t('There are no filters defined.'));
    }
  }
  elseif (isset($fid)) {
    drupal_set_message(t('An error occurred while loading the filter #!fid from the database', array('!fid' => $fid)));
    watchdog('customfilter', 'An error occurred while loading the filter #!fid from the database', array('!fid' => $fid), WATCHDOG_ERROR);
  }

  return $form;
}

function customfilter_test_form_submit($form, &$form_state) {
  $output = '';
  $nid = 0;
  $text = '';

  if (!empty($form_state['values']['node'])) {
    if (($nid = db_result(db_query("SELECT nid FROM {node} WHERE LOWER(title) = LOWER('%s')", $form_state['values']['node']))) && ($node = node_load($nid))) {
      $form_state['storage']['input'] = $text = $node->body;
    }
  }
  elseif (isset($form_state['values']['text'])) {
    $text = $form_state['values']['text'];
  }

  if ($text) {
    $output = customfilter_filter('process', $form_state['values']['filter'], -1, $text);
    $form_state['storage']['output'] = $output;
  }
}

/*****************************************************************************
 * Theme functions.
 ****************************************************************************/

/**
 * Theme for draggable tree of replacement rules
 */
function theme_customfilter_rules_list($form) {
  $has_rules = isset($form['name']) && is_array($form['name']);
  $rows = array();

  if ($has_rules) {
    foreach (element_children($form['name']) as $rid) {
      $row = array();

      $form['prid'][$rid]['#attributes']['class'] = 'rule-prid';
      $form['rid'][$rid]['#attributes']['class'] = 'rule-rid';
      $form['weight'][$rid]['#attributes']['class'] = 'rule-weight';

      $row[] = (
        theme('indentation', $form['level'][$rid]['#value']) .
        drupal_render($form['name'][$rid])
      );
      $row[] = (
        drupal_render($form['rid'][$rid]) .
        drupal_render($form['prid'][$rid]) .
        drupal_render($form['weight'][$rid])
      );
      $row[] = drupal_render($form['enabled'][$rid]);
      $row[] = drupal_render($form['operations'][$rid]['add']);
      $row[] = drupal_render($form['operations'][$rid]['edit']);
      $row[] = drupal_render($form['operations'][$rid]['delete']);

      $rows[] = array(
        'data' => $row,
        'class' => 'draggable',
      );
    }
  }

  if (!count($rows)) {
    $rows[] = array(
      array(
        'data' => t('There are no replacement rules currently defined.'),
        'colspan' => '5',
      )
    );
  }

  $header[] = t('Name');
  $header[] = t('Weight');
  $header[] = array(
    'data' => t('Enabled'),
    'class' => 'customfilter-rules-list-enabled',
  );
  $header[] = array(
    'data' => t('Operations'),
    'colspan' => '3'
  );

  drupal_add_tabledrag('customfilter-rules-tree', 'order', 'sibling', 'rule-weight', NULL, NULL, TRUE);
  drupal_add_tabledrag('customfilter-rules-tree', 'match', 'parent', 'rule-prid', 'rule-prid', 'rule-rid', TRUE);

  return theme('table', $header, $rows, array('id' => 'customfilter-rules-tree')) .
    drupal_render($form);
}

/*****************************************************************************
 * Private functions.
 ****************************************************************************/

/**
 * Map the imported filter ID to a new one.
 */
function _customfilter_map_filter_id($id) {
  static $base = 0, $fids = array();

  if (!$base) {
    $base = (integer) db_result(db_query("SELECT MAX(fid) FROM {customfilter_filters}")) + 1;
  }

  if (!isset($fids[$id])) {
    $fids[$id] = $base++;
  }

  return $fids[$id];
}

/**
 * Map the imported rule ID to a new one.
 */
function _customfilter_map_rule_id($id) {
  static $base = 0, $rids = array();

  if (!$id) {
    return 0;
  }

  if (!$base) {
    $base = (integer) db_result(db_query("SELECT MAX(rid) FROM {customfilter_rules}")) + 1;
  }

  if (!isset($rids[$id])) {
    $rids[$id] = $base++;
  }

  return $rids[$id];
}

/**
 * Helper function for customfilter_export_form.
 * Return the list made of a rule, and all the sub-rules.
 */
function _customfilter_rules_tree(&$rules, $fid, $prid = 0) {
  $result = db_query("SELECT * FROM {customfilter_rules} WHERE fid = %d and prid = %d ORDER BY weight", $fid, $prid);

  while ($rule = db_fetch_array($result)) {
    $rules[] = $rule;
    _customfilter_rules_tree($rules, $fid, $rule['rid']);
  }
}

/**
 * Return the rules tree form.
 */
function _customfilter_rules_tree_form(&$form, &$enabled, $rules, $level = 0) {
  $base = 'admin/settings/customfilter';

  foreach ($rules as $rule) {
    $fid = $rule['fid'];
    $rid = $rule['rid'];

    $enabled['options'][$rid] = '';

    if ($rule['enabled']) {
      $enabled['default'][] = $rid;
    }

    $form['level'][$rid] = array(
      '#type' => 'value',
      '#value' => $level,
    );

    $form['rid'][$rid] = array(
      '#type' => 'hidden',
      '#value' => $rid,
    );

    $form['prid'][$rid] = array(
      '#type' => 'hidden',
      '#default_value' => $rule['prid'],
    );

    $form['name'][$rid] = array(
      '#value' => l($rule['name'], "$base/$fid/$rid/edit"),
    );

    $form['weight'][$rid] = array(
      '#type' => 'weight',
      '#default_value' => $rule['weight'],
    );

    $form['operations'][$rid] = array(
      'add' => array(
        '#value' => l(t('add'), "$base/$fid/$rid/add"),
      ),
      'edit' => array(
        '#value' => l(t('edit'), "$base/$fid/$rid/edit"),
      ),
      'delete' => array(
        '#value' => l(t('delete'), "$base/$fid/$rid/delete"),
      ),
    );

    if ($rule['sub']) {
      _customfilter_rules_tree_form($form, $enabled, $rule['sub'], $level + 1);
    }
  }
}

/**
 * @} End of "addtogroup customfilter".
 */